#!/usr/bin/env python
import subprocess
import sys
from functools import partial
from pathlib import Path

import click
import click_config_file
from streamlit import cli as stcli

try:
    from scml_vis.vendor.quick.quick import gui_option
except:

    def gui_option(x):
        return x


import scml_vis.compiler as compiler
import scml_vis.presenter as presenter
from scml_vis.compiler import VISDATA_FOLDER, has_visdata

click.option = partial(click.option, show_default=True)


@gui_option
@click.group()
def main():
    pass


@main.command(help="Opens the visualizer")
@click.option(
    "-f",
    "--folder",
    type=click.Path(file_okay=False, dir_okay=True),
    default=None,
    help="Folder containing logs of a world or tournament to open. If not given, last runs from SCML or a list of predefined locations will be used",
)
@click.option(
    "-w",
    "--max-worlds",
    default=None,
    type=int,
    help="Maximum number of worlds to keep in the compiled visualization data",
)
def show(folder: Path, max_worlds: int):
    folder = Path(folder) if folder else None
    if folder and not has_visdata(folder):
        try:
            compiler.main(folder, max_worlds)
        except Exception as e:
            print(f"Failed to compile visualization data for {folder}:\nException: {str(e)}")
    if folder:
        sys.argv = ["streamlit", "run", str(Path(__file__).parent / "presenter.py"), str(folder)]
    else:
        sys.argv = ["streamlit", "run", str(Path(__file__).parent / "presenter.py")]
    sys.exit(stcli.main())


@main.command(help="Compiles the data needed for visualization from a given log folder")
@click.argument("folder", type=click.Path(file_okay=False, dir_okay=True))
@click.option(
    "-w",
    "--max-worlds",
    default=None,
    type=int,
    help="Maximum number of worlds to keep in the compiled visualization data",
)
def compile(folder: Path, max_worlds):
    return compiler.main(folder, max_worlds)


@main.command(help="Creates an SQLite dataset and explore it using Datasette")
@click.argument("folder", type=click.Path(file_okay=False, dir_okay=True))
@click.option(
    "-w",
    "--max-worlds",
    default=None,
    type=int,
    help="Maximum number of worlds to keep in the compiled visualization data",
)
def explore(folder: Path, max_worlds):
    folder = Path(folder)
    if not compiler.has_visdata(folder):
        compiler.main(folder, max_worlds)
    dst = str(folder / VISDATA_FOLDER / "dataset.sqlite")
    files = [str(_.absolute()) for _ in (folder / VISDATA_FOLDER).glob("*.csv")]
    subprocess.run(["csvs-to-sqlite"] + files + [dst])
    subprocess.run(["datasette", dst])


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import sys
from pathlib import Path
import traceback

import pandas as pd
import streamlit as st
from streamlit import cli as stcli

import scml_vis.compiler as compiler
from scml_vis.compiler import VISDATA_FOLDER
from scml_vis.utils import add_selector, add_stats_display, add_stats_selector, load_data, plot_network, score_distribution

__all__ = ["main"]

DB_FOLDER = Path.home() / "negmas" / "runsdb"
DB_NAME = "rundb.csv"
BASE_FOLDERS = [
    Path.home() / "negmas" / "logs" / "scml" / "scml2020",
    Path.home() / "negmas" / "logs" / "scml" / "scml2020oneshot",
    Path.home() / "negmas" / "logs" / "scml" / "scml2021oneshot",
    Path.home() / "negmas" / "logs" / "scml" / "scml2021",
    Path.home() / "negmas" / "logs" / "tournaments",
]


def main(folder: Path):
    st.set_page_config(layout="wide")
    if folder is None:
        options = dict(none="none")
        if (DB_FOLDER / DB_NAME).exists():
            data = pd.read_csv(DB_FOLDER / DB_NAME, index_col=None, header=None)
            data: pd.DataFrame
            data = data.iloc[::-1]
            data.columns = ["name", "type", "path"]
            for _, x in data.iterrows():
                options[x["path"]] = f"{x['type'][0]}:{x['name']}"
        add_base = st.sidebar.checkbox("Add Default paths", True)
        if add_base:
            for base in BASE_FOLDERS:
                type_ = base.name == "tournaments"
                for child in base.glob("*"):
                    if not child.is_dir() or not compiler.has_logs(child):
                        continue
                    options[child] = f"{'t' if type_ else 'w'}:{child.name}"

        folder = st.sidebar.selectbox("Select a run", list(options.keys()), format_func=lambda x: options[x])
    if not folder or (isinstance(folder, str) and folder == "none"):
        st.text(
            "Cannot find any folders with logs.\nTry looking in default paths by checking 'Add Default paths' \nin the side bar or start the app with a folder containing log data using -f"
        )
        return
    folder = Path(folder)
    if folder.name != VISDATA_FOLDER:
        folder = folder / VISDATA_FOLDER
    if not folder.exists():
        try:
            do_compile = st.sidebar.button("Compile visualization data?")
            if do_compile:
                try:
                    compiler.main(folder.parent, max_worlds=None)
                except Exception as e:
                    st.write(f"*Failed to compile visualization data for {folder}*\n### Exception:\n{str(e)}")
                    st.write(f"\n### Traceback:\n```\n{traceback.format_exc()}```")
            else:
                st.text("Either press 'Compile visualization data' to view logs of this folder or choose another one.")
                return
        except:
            st.write(f"Folder {folder} contains no logs to use")
        # folder = folder / VISDATA_FOLDER
        # if not folder.exists():
        #     st.write(
        #         f"## SCML Visualizer\nError: No {VISDATA_FOLDER} folder found with visualization data at {str(folder)}"
        #     )
        #     return
    if folder.name != VISDATA_FOLDER:
        folder = folder / VISDATA_FOLDER
    if not folder.exists():
        st.write("Cannot find visualiation data")
        return

    st.write(f"## SCML Visualizer\n{str(folder.parent)}")

    st.sidebar.markdown("## Data Selection")
    tournaments = load_data(folder, "tournaments")
    tournament_expander = st.sidebar.beta_expander("Tournament Selection")
    with tournament_expander:
        selected_tournaments = add_selector(
            st,
            "",
            tournaments.name.unique(),
            key="tournaments",
            none=False,
            default="one",
        )

    worlds = load_data(folder, "worlds")
    worlds = worlds.loc[worlds.tournament.isin(selected_tournaments), :]
    world_expander = st.sidebar.beta_expander("World Selection")
    with world_expander:
        selected_worlds = add_selector(st, "", worlds.name, key="worlds", none=False, default="all")
    worlds = worlds.loc[(worlds.name.isin(selected_worlds)), :]

    agents = load_data(folder, "agents")
    type_expander = st.sidebar.beta_expander("Type Selection")
    with type_expander:
        selected_types = add_selector(st, "", agents.type.unique(), key="types", none=False, default="all")
    agents = agents.loc[(agents.type.isin(selected_types)), :]

    agent_expander = st.sidebar.beta_expander("Agent Selection")
    with agent_expander:
        selected_agents = add_selector(st, "", agents.name.unique(), key="agents", none=False, default="all")

    products = load_data(folder, "product_stats")
    product_expander = st.sidebar.beta_expander("Product Selection")
    with product_expander:
        selected_products = add_selector(
            st, "", products["product"].unique(), key="products", none=False, default="all"
        )

    agents = agents.loc[(agents.type.isin(selected_types)), :]

    nsteps = int(worlds.loc[worlds.name.isin(selected_worlds), "n_steps"].max())
    selected_steps = st.sidebar.slider("Steps", 0, nsteps, (0, nsteps))
    selected_times = st.sidebar.slider("Relative Times", 0.0, 1.0, (0.0, 1.0))

    st.sidebar.markdown("## Figure Selection")
    if len(selected_worlds) == 1:
        fig_type = st.sidebar.selectbox(label="", options=["Time-series", "Networks", "Tables", "Others"], index=1)
    else:
        fig_type = st.sidebar.selectbox(label="", options=["Time-series", "Tables", "Others"], index=1)

    if fig_type == "Time-series":
        runner = display_time_series
    elif fig_type == "Networks":
        runner = display_networks
    elif fig_type == "Tables":
        runner = display_tables
    elif fig_type == "Others":
        runner = display_others
    else:
        st.text("Please choose what type of figures are you interested in")
        return
    products_summary = (
        products.loc[:, [_ for _ in products.columns if _ not in ("step", "relative_time")]]
        .groupby(["tournament", "world", "product"])
        .agg(["min", "max", "mean", "std"])
    )
    products_summary.columns = [f"{a}_{b}" for a, b in products_summary.columns]
    products_summary = products_summary.reset_index()
    data = dict(t=tournaments, w=worlds, a=agents, p=products_summary)

    def filter(x, agent_field_sets):
        if x is None:
            return x
        x = x.loc[(x.world.isin(selected_worlds)), :]
        indx = None
        for fields in agent_field_sets:
            if not fields:
                continue
            indx = x[fields[0]].isin(selected_agents)
            for f in fields[1:]:
                indx = (indx) | (x[f].isin(selected_agents))
        if indx is None:
            return x
        return x.loc[indx, :]

    data["c"] = filter(load_data(folder, "contracts"), [["buyer", "seller"]])
    data["n"] = filter(load_data(folder, "negotiations"), [["buyer", "seller"]])
    data["o"] = filter(load_data(folder, "offers"), [["sender", "receiver"]])
    runner(
        folder,
        selected_worlds,
        selected_products,
        selected_agents,
        selected_types,
        selected_steps,
        selected_times,
        data,
    )


def filter_by_time(x, cols, selected_steps, selected_times):
    indx = None
    for k in cols:
        step_col, time_col = f"{k}step", f"{k}relative_time"
        i = (x[step_col] >= selected_steps[0]) & (x[step_col] <= selected_steps[1])
        i &= (x[time_col] >= selected_times[0]) & (x[time_col] <= selected_times[1])
        if indx is None:
            indx = i
        else:
            indx |= i
    if indx is not None:
        return x.loc[indx, :]
    return x


def display_networks(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
):
    if len(selected_worlds) < 1:
        st.write("No worlds selected. Cannot show any networks")
        return
    if len(selected_worlds) > 1:
        st.write(f"More than one world selected ({len(selected_worlds)}). Cannot show any networks")
        return

    nodes = data["a"].loc[data["a"].world.isin(selected_worlds), :]
    nodes = nodes.to_dict("records")
    added = -data["a"].input_product.min()
    nlevels = data["a"].input_product.max() + 1 + added

    level_max = [0] * (nlevels)
    dx, dy = 10, 10
    for node in nodes:
        l = node["input_product"] + added
        node["pos"] = ((l + 1) * dx, level_max[l] * dy)
        level_max[l] += 1

    nodes = {n["name"]: n for n in nodes}
    nodes["SELLER"] = dict(pos=(0, dy * (level_max[0] // 2)), name="Seller", type="System")
    nodes["BUYER"] = dict(pos=((nlevels + 1) * dx, dy * (level_max[-1] // 2)), name="Seller", type="System")
    what = st.sidebar.selectbox("Category", ["Contracts", "Negotiations"])
    edges, weights = [], []
    per_step = st.checkbox("Show one step only")
    if per_step:
        selected_step = st.slider("Step", selected_steps[0], selected_steps[1], selected_steps[0])
        selected_steps = [selected_step] * 2
    if what == "Contracts":
        src = "c"
    elif what == "Negotiations":
        src = "n"
    else:
        src = "o"
    x = data[src]
    if x is None:
        st.write("Data is not available in the logs.")
        return
    x["total_price"] = x.quantity * x.unit_price
    options = [_[: -len("_step")] for _ in x.columns if _.endswith("_step")]
    if src != "c":
        options.append("step")
    condition_field = st.sidebar.selectbox("Condition", options)
    weight_field = st.sidebar.selectbox("Edge Weight", ["total_price", "unit_price", "quantity", "count"])
    edge_weights = st.sidebar.checkbox("Variable Edge Width", True)
    edge_colors = st.sidebar.checkbox("Variable Edge Colors", True)
    weight_field_name = "quantity" if weight_field == "count" else weight_field
    time_cols = (
        [condition_field + "_step", condition_field + "_relative_time"]
        if condition_field != "step"
        else ["step", "relative_time"]
    )
    x = x.loc[:, [weight_field_name, "seller", "buyer"] + time_cols]
    x = filter_by_time(x, [condition_field + "_" if condition_field != "step" else ""], selected_steps, selected_times)
    x.drop(time_cols, axis=1, inplace=True)
    if weight_field == "unit_price":
        x = x.groupby(["seller", "buyer"]).mean().reset_index()
        x["unit_price"].fillna(0.0, inplace=True)
    elif weight_field == "count":
        x = x.groupby(["seller", "buyer"]).count().reset_index()
        x.rename(columns=dict(quantity="count"), inplace=True)
    else:
        x = x.groupby(["seller", "buyer"]).sum().reset_index()
    for _, d in x.iterrows():
        edges.append((d["seller"], d["buyer"], d[weight_field]))
    node_weight = st.sidebar.selectbox("Node Weight", ["none", "final_score", "cost"])
    st.plotly_chart(plot_network(nodes, edges=edges, node_weights=node_weight, edge_colors=edge_colors, edge_weights=edge_weights))
    if src == "n":
        col1, col2 = st.beta_columns(2)
        seller = col1.selectbox("Seller", x["seller"].unique())
        buyer = col2.selectbox("Buyer", x["buyer"].unique())
        col1, col2 = st.beta_columns(2)
        broken = col1.checkbox("Broken", True)
        timedout = col2.checkbox("Timedout", True)
        options = data["n"].loc[(data["n"].seller == seller) & (data["n"].buyer == buyer), :]
        if not broken:
            options = options.loc[~options.broken]
        if not timedout:
            options = options.loc[~options.timedout]

        options = filter_by_time(options, [""], selected_steps, selected_times)
        neg = st.selectbox(label="Negotiation", options=options.loc[:, "id"].values)
        offers = data["o"]
        offers = offers.loc[offers.negotiation == neg, :].sort_values("round")
        st.dataframe(offers)


def display_tables(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
):
    def show_table(x, must_choose=False):
        selected_cols = st.multiselect(label="Columns", options=x.columns)
        if selected_cols or must_choose:
            st.dataframe(x.loc[:, selected_cols])
        else:
            st.dataframe(x)

    for lbl, k, has_step in (
        ("Tournaments", "t", False),
        ("Worlds", "w", False),
        ("Products", "p", False),
        ("Agents", "a", False),
        ("Contracts", "c", True),
        ("Negotiations", "n", True),
        ("Offers", "o", True),
    ):
        if data[k] is None or not len(data[k]):
            continue
        if st.sidebar.checkbox(label=lbl):
            if has_step:
                x = filter_by_time(
                    data[k], ["signed_", "concluded_"] if k == "c" else [""], selected_steps, selected_times
                )
            else:
                x = data[k]
            show_table(x)
            st.text(f"{len(x)} records found")


def display_time_series(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
):
    settings = st.sidebar.beta_expander("Settings")
    ncols = settings.number_input("N. Columns", min_value=1, max_value=6)
    xvar = settings.selectbox("x-variable", ["step", "relative_time"])
    dynamic = settings.checkbox("Dynamic Figures", value=True)
    sectioned = settings.checkbox("Figure Sections")
    ci_level = settings.selectbox(options=[80, 90, 95], label="CI Level", index=2)
    world_stats, selected_world_stats, combine_world_stats, overlay_world_stats = add_stats_selector(
        folder,
        "world_stats",
        [[("world", selected_worlds), ("step", selected_steps), ("relative_time", selected_times)]],
        xvar=xvar,
        label="World Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "relative_time")
        ],
        default_selector="none",
    )

    product_stats, selected_product_stats, combine_product_stats, overlay_product_stats = add_stats_selector(
        folder,
        "product_stats",
        [[("product", selected_products), ("step", selected_steps), ("relative_time", selected_times)]],
        xvar=xvar,
        label="Product Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "product", "relative_time")
        ],
        default_selector="none",
    )

    type_stats, selected_type_stats, combine_type_stats, overlay_type_stats = add_stats_selector(
        folder,
        "agent_stats",
        [
            [
                ("world", selected_worlds),
                ("type", selected_types),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ]
        ],
        xvar=xvar,
        label="Type Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "relative_time")
        ],
        default_selector="none",
        key="type",
    )

    agent_stats, selected_agent_stats, combine_agent_stats, overlay_agent_stats = add_stats_selector(
        folder,
        "agent_stats",
        [
            [
                ("world", selected_worlds),
                ("name", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ]
        ],
        xvar=xvar,
        label="Agent Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "relative_time")
        ],
        default_selector="none",
    )

    (
        contract_stats_world,
        selected_contract_stats_world,
        combine_contract_stats_world,
        overlay_contract_stats_world,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (World)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="world",
    )

    (
        contract_stats_type,
        selected_contract_stats_type,
        combine_contract_stats_type,
        overlay_contract_stats_type,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Types)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="type",
    )
    (
        contract_stats_agent,
        selected_contract_stats_agent,
        combine_contract_stats_agent,
        overlay_contract_stats_agent,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Agents)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="name",
    )

    def aggregate_contract_stats(stats, ignored_cols):
        cols = [_ for _ in stats.columns if not any(_.endswith(x) for x in ["price", "quantity", "count"])]
        ignored_cols = [_ for _ in cols if _.startswith(ignored_cols)]
        cols = [_ for _ in cols if not _ in ignored_cols]
        allcols = [_ for _ in stats.columns if not _ in ignored_cols]
        # st.text(stats.columns)
        # st.text(allcols)
        # st.text(cols)
        # st.text(len(stats))
        stats = stats.loc[:, allcols].groupby(cols).sum()
        # st.text(len(stats))
        for c in stats.columns:
            if c.endswith("unit_price"):
                base = "_".join(c.split("_")[:-2])
                stats[c] = stats[f"{base}_total_price"] / stats[f"{base}_quantity"]
                stats[c] = stats[c].fillna(0)
        # st.text(len(stats))
        return stats.reset_index()

    (
        contract_stats_buyer_type,
        selected_contract_stats_buyer_type,
        combine_contract_stats_buyer_type,
        overlay_contract_stats_buyer_type,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            # [("world", selected_worlds), ("seller", selected_agents)],
        ],
        xvar=xvar,
        label="Contract Statistics (Buyer Types)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="buyer_type",
    )
    (
        contract_stats_seller_type,
        selected_contract_stats_seller_type,
        combine_contract_stats_seller_type,
        overlay_contract_stats_seller_type,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            # [("world", selected_worlds), ("buyer", selected_agents)],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Seller Types)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="seller_type",
    )
    (
        contract_stats_buyer,
        selected_contract_stats_buyer,
        combine_contract_stats_buyer,
        overlay_contract_stats_buyer,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            # [("world", selected_worlds), ("seller", selected_agents)],
        ],
        xvar=xvar,
        label="Contract Statistics (Buyer)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="buyer",
    )

    (
        contract_stats_seller,
        selected_contract_stats_seller,
        combine_contract_stats_seller,
        overlay_contract_stats_seller,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            # [("world", selected_worlds), ("buyer", selected_agents)],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Seller)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="seller",
    )

    contract_stats_buyer = aggregate_contract_stats(contract_stats_buyer, "seller")
    contract_stats_seller = aggregate_contract_stats(contract_stats_seller, "buyer")
    contract_stats_buyer_type = aggregate_contract_stats(contract_stats_buyer, "seller_type")
    contract_stats_seller_type = aggregate_contract_stats(contract_stats_seller, "buyer_type")

    contract_stats_agent["agent"] = contract_stats_agent["seller"] + "->" + contract_stats_agent["buyer"]
    contract_stats_agent["agent_type"] = contract_stats_agent["seller_type"] + "->" + contract_stats_agent["buyer_type"]
    contract_stats_type["agent"] = contract_stats_type["seller"] + "->" + contract_stats_type["buyer"]
    contract_stats_type["agent_type"] = contract_stats_type["seller_type"] + "->" + contract_stats_type["buyer_type"]

    (
        contract_stats_product,
        selected_contract_stats_product,
        combine_contract_stats_product,
        overlay_contract_stats_product,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Product)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="product",
    )

    cols, start_col = add_stats_display(
        world_stats,
        selected_world_stats,
        combine_world_stats,
        overlay_world_stats,
        ncols=ncols,
        xvar=xvar,
        hue="world",
        title="World Figures",
        sectioned=sectioned,
        cols=None,
        start_col=0,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        product_stats,
        selected_product_stats,
        combine_product_stats,
        overlay_product_stats,
        ncols=ncols,
        xvar=xvar,
        hue="product",
        title="product Figures",
        sectioned=sectioned,
        cols=None,
        start_col=0,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        type_stats,
        selected_type_stats,
        combine_type_stats,
        overlay_type_stats,
        ncols=ncols,
        xvar=xvar,
        hue="type",
        title="Agent Type Figures",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        agent_stats,
        selected_agent_stats,
        combine_agent_stats,
        overlay_agent_stats,
        ncols=ncols,
        xvar=xvar,
        hue="name",
        title="Agent Instance Figures",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_world,
        selected_contract_stats_world,
        combine_contract_stats_world,
        overlay_contract_stats_world,
        ncols=ncols,
        xvar=xvar,
        hue="world",
        title="Trade Figures (World)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_type,
        selected_contract_stats_type,
        combine_contract_stats_type,
        overlay_contract_stats_type,
        ncols=ncols,
        xvar=xvar,
        hue="agent_type",
        title="Trade Figures (Agent Type)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_buyer_type,
        selected_contract_stats_buyer_type,
        combine_contract_stats_buyer_type,
        overlay_contract_stats_buyer_type,
        ncols=ncols,
        xvar=xvar,
        hue="buyer_type",
        title="Trade Figures (Buyer Type)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_seller_type,
        selected_contract_stats_seller_type,
        combine_contract_stats_seller_type,
        overlay_contract_stats_seller_type,
        ncols=ncols,
        xvar=xvar,
        hue="seller_type",
        cols=cols,
        start_col=start_col,
        title="Trade Figures (Seller Type)",
        sectioned=sectioned,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_agent,
        selected_contract_stats_agent,
        combine_contract_stats_agent,
        overlay_contract_stats_agent,
        ncols=ncols,
        xvar=xvar,
        hue="agent",
        cols=cols,
        start_col=start_col,
        title="Trade Figures (Agent Instance)",
        sectioned=sectioned,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_buyer,
        selected_contract_stats_buyer,
        combine_contract_stats_buyer,
        overlay_contract_stats_buyer,
        ncols=ncols,
        xvar=xvar,
        hue="buyer",
        cols=cols,
        start_col=start_col,
        title="Trade Figures (Buyer Instance)",
        sectioned=sectioned,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_seller,
        selected_contract_stats_seller,
        combine_contract_stats_seller,
        overlay_contract_stats_seller,
        ncols=ncols,
        xvar=xvar,
        hue="seller",
        title="Trade Figures (Seller Instance)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_product,
        selected_contract_stats_product,
        combine_contract_stats_product,
        overlay_contract_stats_product,
        ncols=ncols,
        xvar=xvar,
        hue="product",
        title="Trade Figures (Product)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )

def display_others(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
):
    # settings = st.sidebar.beta_expander("Settings")
    # ncols = settings.number_input("N. Columns", min_value=1, max_value=6)
    if st.sidebar.checkbox("Score Distribution", False):
        score_distribution(selected_worlds, selected_agents, selected_types, data)




if __name__ == "__main__":
    import sys

    from streamlit import cli as stcli

    folder = None
    if len(sys.argv) > 1:
        folder = Path(sys.argv[1])

    if st._is_running_with_streamlit:
        main(folder)
    else:
        sys.argv = ["streamlit", "run"] + sys.argv
        sys.exit(stcli.main())



#!/usr/bin/env python
from pathlib import Path
from collections import defaultdict
from pprint import pformat
import random
import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
import seaborn as sns
from typing import Callable
import streamlit as st
import seaborn as sns
import plotly.express as px
import plotly.graph_objects as go
import streamlit as st
import plotly as plotly
from matplotlib import pyplot as plt

DEFAULT_CI_LEVEL = 95
__all__ = [
    "add_selctor",
    "add_stats_selector",
    "add_stats_display_sns",
    "line_with_band",
    "add_stats_display_plotly",
    "add_stats_display",
    "load_data",
    "plot_netowrk",
    "score_distribution"
]


@st.cache
def load_data(folder: Path, name: str):
    file = folder / f"{name}.csv"
    if not file.exists():
        return None
    data = pd.read_csv(file, index_col=None)
    return data


def add_selector(
    parent,
    title,
    content,
    key,
    all=True,
    none=True,
    some=True,
    one=True,
    default="one",
    check=False,
    check2=False,
    default_check=False,
):
    options = []
    indx = 0
    content = sorted(content)
    for a, v in zip((all, some, one, none), ("all", "some", "one", "none")):
        if a:
            options.append(v)
            if v == default:
                indx = len(options) - 1
    parent.text(title)
    col1, col2 = parent.beta_columns([1, 4])
    if check:
        combine = st.checkbox("Combine", value=default_check, key=f"{key}_sel_check")
    else:
        combine = False
    if check2:
        overlay = st.checkbox("Overlay", value=default_check, key=f"{key}_sel_overlay")
    else:
        overlay = False
    with col1:
        selection_type = st.radio(title, options, index=indx, key=f"{key}_sel_type")
    if selection_type == "none":
        return ([], combine, overlay) if check or overlay else []
    if selection_type == "all":
        return (content, combine, overlay) if check or overlay else content
    with col2:
        if selection_type == "some":
            selector = st.multiselect("", content, key=f"{key}_multi")
            return (selector, combine, overlay) if check or overlay else selector
        selector = st.selectbox("", content, key=f"{key}_sel")
        return ([selector], combine, overlay) if check or overlay else [selector]


def add_stats_selector(
    folder,
    file_name,
    filters,
    xvar,
    label,
    default_selector="one",
    choices=None,
    key="",
):
    if label is None:
        label = file_name.split("_")[0] + " Statistics"
        label = label[0].toupper() + label[1:]
    world_stats = load_data(folder, file_name)
    if filters:
        filtered = None
        for fset in filters:
            x = world_stats.copy()
            for field, values in fset:
                if len(x) < 1:
                    break
                if field.endswith("step") or field.endswith("steps") or field.endswith("relative_time"):
                    x = x.loc[(world_stats[field] >= values[0]) & (world_stats[field] <= values[1]), :]
                    continue
                x = x.loc[world_stats[field].isin(values), :]
            if len(x) > 0:
                x = x.index
                if filtered is None:
                    filtered = x
                else:
                    filtered = filtered.union(x)
        if filtered is not None:
            world_stats = world_stats.loc[filtered, :]
        else:
            world_stats = world_stats.loc[[False] * len(world_stats), :]
        # world_stats = pd.concat(filtered, ignore_index=False)

    if choices is None:
        choices = [_ for _ in world_stats.columns if _ not in ("step", "relative_time")]
    elif isinstance(choices, Callable):
        choices = choices(world_stats)

    world_stats_expander = st.sidebar.beta_expander(label)
    with world_stats_expander:
        selected_world_stats, combine_world_stats, overlay_world_stats = add_selector(
            st,
            "",
            choices,
            key=f"{file_name}_{key}",
            none=True,
            default=default_selector,
            check=True,
            check2=True,
            default_check=xvar == "step",
        )
    return world_stats, selected_world_stats, combine_world_stats, overlay_world_stats


def add_stats_display_sns(
    stats,
    selected,
    combine,
    overlay,
    hue,
    xvar,
    cols=None,
    start_col=0,
    ncols_effective=0,
    ci_level=DEFAULT_CI_LEVEL,
):
    displayed = 0
    if overlay:
        fig, ax = plt.subplots()
    for i, field in enumerate(selected):
        if not overlay:
            fig, ax = plt.subplots()
            plt.title(field)
        sns.lineplot(
            data=stats,
            x=xvar,
            y=field,
            label=field if overlay else None,
            ax=ax,
            hue=hue if not combine else None,
            style=None,
            ci=ci_level,
        )
        if not overlay:
            with cols[(i + start_col) % ncols_effective]:
                displayed += 1
                st.pyplot(fig)
    if overlay:
        with cols[(displayed + start_col) % ncols_effective]:
            displayed += 1
            st.pyplot(fig)
    return cols, displayed + start_col


def line_with_band(fig, stats, xvar, yvar, color, i, color_val=None, ci_level=DEFAULT_CI_LEVEL):
    if color is not None:
        for i, v in enumerate(stats[color].unique()):
            fig = line_with_band(
                fig, stats.loc[stats[color] == v, :], xvar, yvar, None, i, color_val=v, ci_level=ci_level
            )
            fig.update_layout(yaxis_title=yvar)
        return fig
    colors = px.colors.qualitative.Plotly
    if color:
        stats = stats.groupby([xvar, color]).agg(["mean", "std", "count"])
    else:
        stats = stats.groupby([xvar]).agg(["mean", "std", "count"])
    stats.columns = [f"{a}_{b}" for a, b in stats.columns]
    for c in stats.columns:
        if not c.endswith("mean"):
            continue
        base = c[: -len("_mean")]
        if not f"{base}_std" in stats.columns or not f"{base}_count" in stats.columns:
            stats[f"{base}"] = stats[c]
            stats = stats.drop([c])
            continue
        stats[f"{base}_ci_hi"] = stats[c]
        stats[f"{base}_ci_lo"] = stats[c]
        indx = stats[f"{base}_count"] > 0
        stats.loc[indx, f"{base}_ci_hi"] = stats.loc[indx, c] + (1 + ci_level / 100.0) * stats.loc[
            indx, f"{base}_std"
        ] / stats.loc[indx, f"{base}_count"].apply(np.sqrt)
        stats.loc[indx, f"{base}_ci_lo"] = stats.loc[indx, c] - (1 + ci_level / 100.0) * stats.loc[
            indx, f"{base}_std"
        ] / stats.loc[indx, f"{base}_count"].apply(np.sqrt)
        stats[f"{base}"] = stats[c]
        stats = stats.drop([c, f"{base}_std", f"{base}_count"], axis=1)
    stats = stats.reset_index()
    x, y, hi, lo = stats[xvar], stats[f"{yvar}"], stats[f"{yvar}_ci_hi"], stats[f"{yvar}_ci_lo"]
    if fig is None:
        fig = go.Figure()
    if not isinstance(color_val, str):
        color_val = str(color_val) 
    yname = yvar if not color_val else f"{color_val}:{yvar}" if ":" not in color_val else color_val
    fig.add_trace(go.Scatter(x=x, y=y, name=yname, line_color=colors[i % len(colors)]))
    clr = str(tuple(plotly.colors.hex_to_rgb(colors[i % len(colors)]))).replace(" ", "")
    clr = f"rgba{clr[:-1]},0.2)"
    fig.add_trace(
        go.Scatter(
            x=x.tolist() + x[::-1].tolist(),  # x, then x reversed
            y=(hi).tolist() + (lo[::-1]).tolist(),  # upper, then lower reversed
            fill="toself",
            fillcolor=clr,
            line=dict(color="rgba(255,255,255,0)"),
            hoverinfo="skip",
            showlegend=False,
            name=f"{yname}_{ci_level}ci",
        )
    )
    fig.update_layout(xaxis_title=xvar)
    return fig


def add_stats_display_plotly(
    stats,
    selected,
    combine,
    overlay,
    hue,
    xvar,
    cols=None,
    start_col=0,
    ncols_effective=0,
    ci_level=DEFAULT_CI_LEVEL,
):
    displayed, fig = 0, None
    if overlay:
        fig = go.Figure()
    # allcols = [xvar] + ([hue] if hue else []) + selected
    # stats = stats.loc[:, allcols]
    if combine:
        stats = stats.loc[:, [_ for _ in stats.columns if _ != hue or _ in selected]]
    # st.text([xvar, hue, selected])
    # st.table(stats.loc[(stats.step==0) & (stats.agent=="03SyR@1->05Dec@1"), :])
    for i, field in enumerate(selected):
        if not overlay:
            fig = line_with_band(None, stats, xvar, field, color=hue if not combine else None, i=i, ci_level=ci_level)
            fig.update_layout(showlegend=not combine)
            if combine:
                fig.update_layout(yaxis_title=field)
            with cols[(i + start_col) % ncols_effective]:
                displayed += 1
                st.plotly_chart(fig)
            continue
        col_name = "value" if len(selected) > 1 else field.split(":")[-1]
        fig = line_with_band(fig, stats, xvar, field, color=None, i=i, ci_level=ci_level)
        fig.update_layout(yaxis_title=col_name)
        fig.update_layout(showlegend=len(selected) > 1 or not combine)
    if overlay:
        with cols[(displayed + start_col) % ncols_effective]:
            displayed += 1
            st.plotly_chart(fig)
    return cols, displayed + start_col


def add_stats_display(
    stats,
    selected,
    combine,
    overlay,
    hue,
    xvar,
    ncols,
    cols=None,
    start_col=0,
    title=None,
    sectioned=False,
    dynamic=False,
    ci_level=DEFAULT_CI_LEVEL,
):
    if "bankrupt" in stats.columns:
        stats["bankrupt"] = stats.bankrupt.astype(int)
    add_section = False
    if sectioned:
        start_col, cols = 0, None
        if title:
            add_section = True
    if len(selected) < 1:
        return cols, start_col
    allcols = [xvar] + ([hue] if hue else []) + selected
    stats = stats.loc[:, allcols]
    if add_section:
        st.markdown(f"### {title}")
    if sectioned or cols is None:
        ncols_effective = max(1, min(len(stats), ncols))
        cols = st.beta_columns(ncols_effective)
    else:
        ncols_effective = len(cols)
    displayed = 0
    if overlay and not combine:
        data = stats.loc[:, [hue, xvar] + selected]
        data = data.melt(id_vars=[xvar, hue], value_vars=selected)
        data["variable"] = data[hue].astype(str) + ":" + data["variable"]
        if len(selected) == 1:
            col_name = selected[0].split(":")[-1]
            data = data.rename(columns=dict(value=col_name))
        else:
            col_name = "value"
        if dynamic:
            presenter = st.plotly_chart
            fig = line_with_band(None, data, xvar, col_name, color="variable", i=0, ci_level=ci_level)
            fig.update_layout(yaxis_title=col_name)
            fig.update_layout(showlegend=len(selected) > 1 or not combine)
        else:
            presenter = st.pyplot
            fig, ax = plt.subplots()
            sns.lineplot(data=data, x=xvar, y=col_name, hue="variable", ax=ax, style=None)
        with cols[(displayed + start_col) % ncols_effective]:
            displayed += 1
            presenter(fig)
        return cols, displayed + start_col
    runner = add_stats_display_plotly if dynamic else add_stats_display_sns
    cols, end_col = runner(
        stats,
        selected,
        combine,
        overlay,
        hue,
        xvar,
        cols=cols,
        start_col=start_col,
        ncols_effective=ncols_effective,
        ci_level=ci_level,
    )
    return cols, end_col


def plot_network(nodes, node_weights=None, color_title=None, edges=[], title="", edge_weights=True, edge_colors=True):
    edge_x = []
    edge_y = []
    annotations = []
    min_width, max_width = 1, 7
    weights = []
    colors = []
    for edge in edges:
        # st.text((nodes[edge[0]]["name"], nodes[edge[1]]["name"]))
        x0, y0 = nodes[edge[0]]["pos"]
        x1, y1 = nodes[edge[1]]["pos"]
        if edge_weights or edge_colors:
            edge_x.append([x0, x1, None])
            edge_y.append([y0, y1, None])
        else:
            edge_x += [x0, x1, None]
            edge_y += [y0, y1, None]
        fraction = random.random() * 0.4 + 0.2
        slope = (y1-y0) / (x1 - x0)
        dx = fraction * (x1 - x0)
        x = x0 +dx 
        y = slope * dx + y0
        # x = min(x0, x1) + fraction * (max(x0, x1) - min(x0, x1))
        # y = min(y0, y1) + fraction * (max(y0, y1) - min(y0, y1))
        if edge_colors:
            clr = tuple([random.randint(0, 255), random.randint(0, 255), random.randint(0, 255)])
            clr = f"rgb{clr}"
        else:
            clr = "#888"
        annotations.append((x, y, edge[2]))
        weights.append(edge[2])
        colors.append(clr)

    if weights and len(weights):
        mn, mx = min(weights), max(weights)
        if mx == mn:
            weights = [1]*len(weights)
        else:
            weights = [(_-mn) *(max_width - min_width)/ (mx-mn) + min_width for _ in weights]

    edge_traces = []
    if edge_weights or edge_colors:
        for x, y, w, clr in zip(edge_x, edge_y, weights, colors):
            if np.isnan(w):
                continue
            # "#888"
            line = dict()
            if edge_colors:
                line["color"] = clr
            else:
                line["color"] = "#888"
            if edge_weights:
                line["width"] = w
            edge_traces.append(go.Scatter(x=x, y=y, line=line, hoverinfo="text", mode="lines"))
    else:
        edge_traces.append(go.Scatter(x=edge_x, y=edge_y, hoverinfo="text", mode="lines"))

    node_x = []
    node_y = []
    node_info = []
    node_w = []
    for node in nodes:
        n= nodes[node]
        x, y = n["pos"]
        node_x.append(x)
        node_y.append(y)
        node_info.append((n.get('cost', 0), n.get('final_score', 0)))
        node_w.append(n.get(node_weights, 1))
    node_trace = go.Scatter(
        name="",
        x=node_x,
        y=node_y,
        mode="markers+text",
        customdata=node_info,
        textposition="top center",
        hovertemplate="cost:%{customdata[0]:.1f}<br>score:%{customdata[1]:.3f} ",
        marker=dict(
            showscale=True,
            # colorscale options
            #'eys' | 'YlBu' | 'eens' | 'YlOrRd' | 'Bluered' | 'RdBu' |
            #'Reds' | 'Blues' | 'Picnic' | 'Rainbow' | 'Portland' | 'Jet' |
            #'Hot' | 'Blackbody' | 'Earth' | 'Electric' | 'Viridis' |
            # colorscale=plotly.colors.col
            colorscale = "plotly3",
            reversescale=True,
            color=node_w,
            size=30,
            colorbar=dict(thickness=12, title=color_title, xanchor="left", titleside="right") if color_title else None,
            line_width=2,
        ),
    )

    node_text = []
    for node in nodes:
        node_text.append(str(node))
    node_trace.marker.color = node_w
    node_trace.text = node_text

    fig = go.Figure(
        data=edge_traces + [node_trace],
        layout=go.Layout(
            title=title,
            titlefont_size=16,
            showlegend=False,
            hovermode="closest",
            margin=dict(b=20, l=5, r=5, t=40),
            # annotations=[ dict(
            #     text="Python code: <a href='https://plotly.com/ipython-notebooks/network-graphs/'> https://plotly.com/ipython-notebooks/network-graphs/</a>",
            #     showarrow=False,
            #     xref="paper", yref="paper",
            #     x=0.005, y=-0.002 ) ],
            xaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
            yaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
        ),
    )

    for ((x, y, txt), clr) in zip(annotations, colors):
        if edge_colors:
            fig.add_annotation(x=x, y=y, text=txt, showarrow=False, yshift=10, font=dict(color=clr))
        else:
            fig.add_annotation(x=x, y=y, text=txt, showarrow=False, yshift=10)
    # st.write(st.get_option("theme"))
    return fig


def score_distribution(selected_worlds, selected_agents, selected_types, data):
    # st.write(data["a"])
    # st.write(data["a"].groupby(["world", "type", "input_product"])["final_score"].count())
    expander = st.beta_expander("Score Distribution")
    col1, col2, col3, col4 = expander.beta_columns(4)
    is_type = col1.checkbox("Agent Types", value=True, key=f"is_type_check")
    independent_levels = col2.checkbox("Independent Production Levels", value=True, key=f"is_independent_levels")
    no_default = col3.checkbox("No Default Agents", value=True, key=f"no_default_agents")
    selected = selected_types if is_type else selected_agents
    col = "type" if is_type else "name"
    data = data["a"].loc[data["a"].world.isin(selected_worlds) , [col, "world", "tournament", "final_score", "input_product", "is_default"]]
    if no_default:
        data = data.loc[~data.is_default, :]
    data = data.drop("is_default", axis=1)
    data.columns = ["agent", "world", "tournament", "final_score", "level"]
    data = data.loc[data.agent.isin(selected), :]
    if len(data.tournament.unique()) > 0:
        data["world"] = data.tournament + "-" + data.world
    if independent_levels:
        data["agent"] = data.agent + "@" + data.level.astype(str)
    data.drop("tournament", axis=1)
    data.drop("level", axis=1)
    agnts = sorted(data["agent"].unique().tolist())
    n = len(agnts)
    map = dict(zip(agnts, range(n)))
    img = np.zeros((n, n))
    count_img = np.zeros((n, n))
    world_agents = defaultdict(list)
    scores = defaultdict(list)
    counts = defaultdict(list)
    for indx, x in data.iterrows():
        world_agents[x.world].append(x.agent)
        scores[(x.world, x.agent)] = x.final_score
        counts[(x.world, x.agent)] = x.final_score
    for (world, agent), score in scores.items():
        for opponent in world_agents[world]:
            if opponent == agent:
                continue
            img[map[agent], map[opponent]] += score
    for (world, agent), score in counts.items():
        for opponent in world_agents[world]:
            if opponent == agent:
                continue
            count_img[map[agent], map[opponent]] += 1

    expander.write("## Scores")
    col1, col2 = expander.beta_columns(2)
    fig = px.imshow(img, x = agnts, y = agnts)
    col1.plotly_chart(fig)
    col2.plotly_chart(px.bar(data, x="agent", y="final_score"))

    expander.write("## Counts")
    col1, col2 = expander.beta_columns(2)
    fig = px.imshow(count_img, x = agnts, y = agnts)
    col1.plotly_chart(fig)
    scores = data.groupby("agent")["final_score"].count().reset_index()
    scores = scores.rename(columns=dict(final_score="count"))
    col2.plotly_chart(px.bar(scores, x="agent", y="count"))



#!/usr/bin/env python
from typing import Optional
import itertools
import pandas as pd
import numpy as np
from typing import Iterable
from pathlib import Path
import json
from pathlib import Path
import sys

__all__ = ["has_visdata", "has_logs", "main", "VISDATA_FOLDER"]

VISDATA_FOLDER = "_visdata"
# tournament files
SCORES_FILE = "scores.csv"
CONFIGS_FILE = "assigned_configs.json"  # has n_steps, __dir_name
# WORLD_STATS_FILE = "stats.csv" # has path and world

TOURNAMENT_REQUIRED = [SCORES_FILE, CONFIGS_FILE]

# single world files
CONTRACTS_FILE = ("contracts.csv", "contracts_full_info.csv")
NEGOTIATIONS_FILE = "negotiations.csv"
INFO_FILE = "info.json"
BREACHES_FILE = "breaches.csv"
AGENTS_FILE = "agents.csv"
PARAMS_FILE = "params.csv"
STATS_FILE = "stats.csv"
AGENTS_JSON_FILE = "agents.json"
WORLD_REQUIRED = [STATS_FILE]
MAXWORLDS = float("inf")

PATHMAP = dict()


def has_logs(folder: Path, check_tournament=True, check_world=True) -> bool:
    if check_world:
        for f in WORLD_REQUIRED:
            if not (folder/ f).exists():
                return False
        return True
    if check_tournament:
        for f in TOURNAMENT_REQUIRED:
            if not (folder/ f).exists():
                return False
        return True
    return True

def adjust_type_names(df):
    if df is None or len(df) == 0:
        return df
    for c in df.columns:
        if not c.endswith("type"):
            continue
        df[c] = df[c].str.split(".").str[0] + "." + df[c].str.split(".").str[-1]
    return df


def nonzero(f):
    return f.exists() and f.stat().st_size > 0


def get_folders(base_folder, main_file, required):
    return [
        _.parent
        for _ in base_folder.glob(f"**/{main_file}")
        if all(nonzero(f) for f in [_.parent / n for n in required])
    ]


def is_tournament(base_folder):
    return all(nonzero(f) for f in [base_folder / n for n in TOURNAMENT_REQUIRED])


def is_world(base_folder):
    return all(nonzero(f) for f in [base_folder / n for n in WORLD_REQUIRED])


def get_torunaments(base_folder):
    return get_folders(base_folder, main_file=CONFIGS_FILE, required=TOURNAMENT_REQUIRED)


def get_worlds(base_folder):
    return get_folders(base_folder, main_file=AGENTS_FILE, required=WORLD_REQUIRED)


def parse_tournament(path, t_indx, base_indx):
    configs = json.load(open(path / CONFIGS_FILE))
    if not configs:
        return None, None, None
    scores = pd.read_csv(path / SCORES_FILE).to_dict("records") # typing: none
    if not scores:
        return None, None, None
    worlds = []
    world_indx = dict()
    world_names = set()
    agents = []
    for i, _ in enumerate(configs):
        if i > MAXWORLDS:
            break
        c = _[0]
        world_names.add(c["world_params"]["name"])
        p = c["__dir_name"]
        for k, v in PATHMAP.items():
            p = p.replace(k, v)
        worlds.append(
            dict(
                id=i + base_indx,
                path=p,
                name=c["world_params"]["name"],
                n_steps=c["world_params"]["n_steps"],
                n_processes=c["world_params"]["n_processes"],
                n_agents=len(c["is_default"]),
                tournament=path.name,
                tournament_indx=t_indx,
            )
        )
        world_indx[worlds[-1]["id"]] = worlds[-1]["name"]
        _, wa = get_basic_world_info(Path(p), path.name)
        agents.append(wa)

    agents = pd.concat(agents)

    # for i, s in enumerate(scores):
    #     if s["world"] not in (world_names):
    #         continue
    #     agents.append(
    #         dict(
    #             id=i + base_indx,
    #             name=s["agent_id"],
    #             type=s["agent_type"],
    #             # id=s["agent_id"],
    #             final_score=s["score"],
    #             world=s["world"],
    #             world_id=world_indx.get(s["world"], None),
    #         )
    #     )
    # agents = pd.DataFrame.from_records(agents)
    return worlds, agents


def parse_world(path, tname, wname, nsteps, agents, w_indx, base_indx):
    stats = pd.read_csv(path / STATS_FILE, index_col=0)
    # try:
    #     winfo = json.load(open(path / INFO_FILE))
    # except:
    #     winfo = None
    if (path / AGENTS_JSON_FILE).exists():
            scored_agents = set(agents["name"].to_list())
            ag_dict = json.load(open(path / AGENTS_JSON_FILE))
            for k, v in ag_dict.items():
                if k in ("BUYER", "SELLER"):
                    v["final_score"] = float("nan")
                elif k in scored_agents:
                    a, b = stats[f"score_{k}"].values[-1], float(agents.loc[agents["name"]==k, "final_score"].values)
                    assert a == b
                    v["final_score"] = float(agents.loc[agents["name"]==k, "final_score"].values)
                else:
                    v["final_score"] = stats[f"score_{k}"].values[-1]
            world_agents = pd.DataFrame.from_records(list(ag_dict.values()))
            world_agents["official_score"] = world_agents["name"].isin(scored_agents)
    else:
        world_agents = agents
        world_agents["official_score"] = True
    contracts = pd.DataFrame(
        data=[],
        columns=[
            "id",
            "seller_name",
            "buyer_name",
            "seller_type",
            "buyer_type",
            "delivery_time",
            "quantity",
            "unit_price",
            "signed_at",
            "nullified_at",
            "concluded_at",
            "signatures",
            "issues",
            "seller",
            "buyer",
            "product_name",
            "n_neg_steps",
            "negotiation_id",
            "executed_at",
            "breaches",
            "erred_at",
            "dropped_at",
        ],
    )
    negotiations = pd.DataFrame(
        data=[],
        columns=[
            "id",
            "seller",
            "buyer",
            "seller_type",
            "buyer_type",
            "delivery_step",
            "delivery_relative_time",
            "quantity",
            "unit_price",
            "step",
            "relative_time",
            "timedout",
            "broken",
            "issues",
            "caller",
            "product",
            "rounds",
            "world",
            "tournament",
            "min_quantity",
            "max_quantity",
            "min_delivery_step",
            "max_delivery_step",
            "min_delivery_relative_time",
            "max_delivery_relative_time",
            "min_unit_price",
            "max_unit_price",
        ],
    )

    offers_list = []
    offers = pd.DataFrame(
        data=[],
        columns=[
            "sender",
            "receiver",
            "sender_type",
            "receiver_type",
            "delivery_step",
            "quantity",
            "unit_price",
            "step",
            "negotiation",
            "relative_time",
            "timedout",
            "broken",
            "caller",
            "product",
            "world",
            "tournament",
            "round",
        ],
    )
    for cname in CONTRACTS_FILE:
        if nonzero(path / cname):
            contracts = pd.read_csv(path / cname, index_col=0)
    if nonzero(path / NEGOTIATIONS_FILE):
        negotiations = pd.read_csv(path / NEGOTIATIONS_FILE, index_col=0)
        negotiations = negotiations.loc[
            :,
            [
                "id",
                "agreement",
                "step",
                "timedout",
                "broken",
                "issues",
                "buyer",
                "seller",
                "caller",
                "product",
                "step",
                "offers",
                "ended_at",
            ],
        ]
        negotiations = negotiations.rename(columns=dict(step="rounds"))
        negotiations = negotiations.rename(columns=dict(ended_at="step"))
        negotiations["offers"] = negotiations.offers.apply(lambda x: eval(x) if isinstance(x, str) else x)
        negotiations["rounds"] = negotiations["rounds"] - 1
        for indx, neg in negotiations.iterrows():
            d = dict(
                sender=None,
                receiver=None,
                delivery_step=None,
                quantity=None,
                unit_price=None,
                round=None,
                index=None,
                step=neg["step"],
                timedout=neg["timedout"],
                broken=neg["broken"],
                caller=neg["caller"],
                product=neg["product"],
                world=wname,
                tournament=tname,
            )
            if not neg["offers"]:
                continue
            offers = eval(neg["offers"]) if isinstance(neg["offers"], str) else neg["offers"]
            current_offers = []
            for k, vs in offers.items():
                for r, v in enumerate(vs):
                    dd = {**d}
                    dd["sender"] = k
                    dd["negotiation"] = neg["id"]
                    dd["receiver"] = neg["seller"] if k == neg["buyer"] else neg["buyer"]
                    dd["round"] = r
                    if v:
                        dd["quantity"] = v[0]
                        dd["delivery_step"] = v[1]
                        dd["unit_price"] = v[2]
                    else:
                        dd["quantity"] = None
                        dd["delivery_step"] = None
                        dd["unit_price"] = None

                    current_offers.append(dd)
            for indx, v in enumerate(current_offers):
                v["index"] = indx
            offers_list += current_offers

        atmap = dict(zip(world_agents["name"].to_list(), world_agents["type"].to_list()))
        # atmap = dict(zip(all_agents["name"].to_list(), all_agents["type"].to_list()))
        # negotiations.agreement[negotiations.agreement.isna(), "agreement"] = None
        for c in ["quantity", "delivery_step", "unit_price"]:
            negotiations[c] = None

        def lst(x):
            return (
                [-1, -1, -1]
                if not x or (isinstance(x, str) and x.lower() == "none")
                else list(eval(x))
                if isinstance(x, str)
                else list(x)
            )

        negotiations.agreement = negotiations.agreement.apply(lst)
        agreements = pd.DataFrame(negotiations.agreement.tolist(), index=negotiations.index)
        agreements.columns = ["quantity", "delivery_step", "unit_price"]
        for c in ["quantity", "delivery_step", "unit_price"]:
            negotiations[c] = agreements[c]
            negotiations.loc[negotiations.loc[:, c] < 0, c] = float("nan")

        def do_map(x):
            return atmap[x]

        negotiations["buyer_type"] = negotiations.buyer.apply(do_map)
        negotiations["seller_type"] = negotiations.seller.apply(do_map)
        negotiations["world"] = wname
        negotiations["tournament"] = tname
        negotiations = negotiations.drop("agreement", axis=1)

        def get_issue_ranges(x):
            if isinstance(x, str):
                x = eval(x)
            if not isinstance(x, Iterable):
                return x
            y = []
            for s in x:
                mn, mx = eval(s.split(":")[-1].strip())
                y += [mn, mx]
            return y

        negotiations = negotiations.reset_index(None)
        issues = negotiations.issues.apply(get_issue_ranges)
        issues = pd.DataFrame(
            issues.to_list(),
            columns=[
                "min_quantity",
                "max_quantity",
                "min_delivery_step",
                "max_delivery_step",
                "min_unit_price",
                "max_unit_price",
            ],
        )
        negotiations = pd.concat((negotiations, issues), axis=1)
        if len(negotiations):
            negotiations.drop("issues", axis=1, inplace=True)
            for c in negotiations.columns:
                if not c.endswith("step") and not c.endswith("steps"):
                    continue
                negotiations.loc[negotiations[c] < 0, c] = float("nan")
                negotiations[c.replace("step", "relative_time")] = negotiations[c] / nsteps if nsteps else 0.0
        offers = pd.DataFrame.from_records(offers_list)
        if len(offers):
            offers["receiver_type"] = offers.receiver.apply(do_map)
            offers["sender_type"] = offers.sender.apply(do_map)
            for c in offers.columns:
                if not c.endswith("step") and not c.endswith("steps"):
                    continue
                offers.loc[offers[c] < 0, c] = float("nan")
                offers[c.replace("step", "relative_time")] = offers[c] / nsteps if nsteps else 0.0

    contracts = contracts.loc[
        :,
        [
            "id",
            "seller_type",
            "buyer_type",
            "delivery_time",
            "quantity",
            "unit_price",
            "signed_at",
            "nullified_at",
            "concluded_at",
            # "issues",
            "seller",
            "buyer",
            "product_name",
            "n_neg_steps",
            "negotiation_id",
            "executed_at",
            "breaches",
            "erred_at",
            "dropped_at",
        ],
    ]
    contracts = contracts.rename(
        columns=dict(
            delivery_time="delivery_step",
            negotiation_id="negotiation",
            product_name="product",
        )
    )
    contracts["product"] = contracts["product"].str[1:].astype(int)
    contracts.columns = [_.replace("_at", "_step") if _.endswith("_at") else _ for _ in contracts.columns]
    # contracts["step"] = contracts["concluded_step"]
    contracts["breached_step"] = contracts["executed_step"]
    contracts["breaches"] = contracts.breaches.astype(str)
    contracts.loc[(contracts.signed_step >= 0) & (contracts.executed_step < 0), "breached_step"] = -1
    contracts.loc[
        (contracts.signed_step >= 0) & (contracts.breaches.str.len() > 0),
        "breached_step",
    ] = -1
    contracts["world"] = wname
    contracts["tournament"] = tname
    # step_cols = (
    #     "",
    #     "concluded_",
    #     "signed_",
    #     "executed_",
    #     "erred_",
    #     "dropped_",
    #     "nullified_",
    #     "breached_",
    # )
    for c in contracts.columns:
        if not c.endswith("step") and not c.endswith("steps"):
            continue
        contracts.loc[contracts[c] < 0, c] = float("nan")
        contracts[c.replace("step", "relative_time")] = contracts[c] / nsteps if nsteps else 0.0

    contract_stats, negotiation_stats = [], []
    non_step_cols = [
        "unit_price",
        "quantity",
        "product",
        "buyer",
        "seller",
        "buyer_type",
        "seller_type",
    ]

    def calc_qp(x):
        c = len(x)
        s = (x["quantity"] * x["unit_price"]).sum()
        q = x["quantity"].sum()
        if q:
            return q, c, s / q
        return 0, 0, 0.0

    for step in range(nsteps):
        s = None
        for c in contracts.columns:
            if not c.endswith("step"):
                continue
            base = c.replace("step", "").replace("_", "")
            qp = contracts.loc[contracts[c] == step, non_step_cols].groupby(non_step_cols[2:]).apply(calc_qp)
            if not isinstance(qp, pd.DataFrame):
                qp = pd.DataFrame(data=qp, index=qp.index)
            if len(qp) < 1:
                continue
            # breakpoint()
            qp.columns = ["qp"]
            # breakpoint()
            qp = pd.DataFrame(qp["qp"].tolist(), index=qp.index)
            qp = qp.rename(columns={0: base + "_quantity", 1: base + "_count", 2: base + "_unit_price"})
            # qp = qp.reset_index()
            qp["step"] = step
            qp.set_index(["step"], append=True, inplace=True)
            qp = qp.melt(
                value_vars=[base + "_quantity", base + "_count", base + "_unit_price"],
                ignore_index=False,
            )
            if s is None:
                s = qp
            else:
                s = pd.concat((s, qp), ignore_index=False, axis=0)
        if s is not None:
            contract_stats.append(s.reset_index())
    # contract_stats = pd.DataFrame.from_records(contract_stats)
    contract_stats = pd.concat(contract_stats, ignore_index=True)
    contract_stats = contract_stats.pivot(
        index=[_ for _ in contract_stats.columns if _ not in ("variable", "value")],
        columns="variable",
        values="value",
    ).reset_index()
    contract_stats["relative_time"] = (contract_stats["step"] / nsteps) if nsteps else 0.0
    contract_stats = contract_stats.fillna(0)
    contract_stats["world"] = wname
    contract_stats["tournament"] = tname
    for col in [_ for _ in contract_stats.columns if _.endswith("unit_price")]:
        field = col.split("_")[0]
        if f"{field}_quantity" in contract_stats.columns:
            contract_stats[f"{field}_total_price"] = (
                contract_stats[f"{field}_quantity"] * contract_stats[f"{field}_unit_price"]
            )

    for c in contract_stats.columns:
        if c.endswith("quantity"):
            contract_stats[c] = contract_stats[c].astype(int)
        if c.endswith("count"):
            contract_stats[c] = contract_stats[c].astype(int)
        if c.endswith("unit_price"):
            contract_stats[c] = contract_stats[c].astype(float)

    # contract_stats.drop_duplicates(inplace=True, subset=["buyer", "seller", "step"])

    if nonzero(path / BREACHES_FILE):
        breaches = pd.read_csv(path / BREACHES_FILE, index_col=0)
    else:
        breaches = pd.DataFrame(
            data=[],
            columns=[
                "contract",
                "contract_id",
                "type",
                "level",
                "id",
                "perpetrator",
                "perpetrator_type",
                "victims",
                "step",
                "resolved",
            ],
        )
    breaches = breaches.loc[
        :,
        [
            "step",
            "contract_id",
            "type",
            "level",
            "id",
            "perpetrator",
            "perpetrator_type",
            "victims",
            "resolved",
        ],
    ]
    breaches = breaches.rename(columns=dict(contract_id="contract"))
    breaches["victim"] = breaches["victims"].apply(lambda x: x[0] if not isinstance(x, str) else eval(x)[0])
    # contracts["relative_time"] = breaches["step"] / nsteps
    breaches.drop(["victims"], axis=1)
    agent_names = world_agents["name"].unique()
    # inventory_{}_input", output
    product_stat_names = ["trading_price", "sold_quantity", "unit_price"]
    products = set([_.split("_")[-1] for _ in stats.columns if any(_.startswith(p) for p in product_stat_names)])
    agent_stat_names = [
        "balance",
        "productivity",
        "bankrupt",
        "score",
        "assets",
        "inventory-input",
        "inventory-output",
        "spot_market_loss",
        "spot_market_quantity",
    ]
    agents_info, product_info = [], []
    world_stat_names = list(
        str(_)
        for _ in stats.columns
        if not any(_.startswith(n.split("-")[0]) for n in (agent_stat_names + product_stat_names))
    )
    for aname in agent_names:
        colnames = []
        ainfo = world_agents.loc[world_agents["name"] == aname, ["type", "final_score"]].to_dict("records")[0]
        for n in agent_stat_names:
            ns = n.split("-")
            if len(ns) > 1:
                col_name = f"{ns[0]}_{aname}_{ns[-1]}"
            else:
                col_name = f"{n}_{aname}"
            if col_name not in stats.columns:
                continue
            colnames.append(col_name)
        x = stats.loc[:, colnames].reset_index().rename(columns=dict(index="step"))
        x.columns = [_ if aname not in _ else _.replace(f"_{aname}", "").replace("-", "_") for _ in x.columns]
        if len(x):
            x["relative_time"] = x["step"] / nsteps
        else:
            x["relative_time"] = 0.0
        x["name"] = aname
        x["world"] = wname
        x["tournament"] = tname
        for k, v in ainfo.items():
            x[k] = v
        agents_info.append(x)

    for p in products:
        colnames = []
        for n in product_stat_names:
            col_name = f"{n}_{p}"
            colnames.append(col_name)
        x = stats.loc[:, colnames].reset_index().rename(columns=dict(index="step"))
        x.columns = ["_".join(_.split("_")[:-1]) if _.endswith(f"_{p}") else _ for _ in x.columns ]
        if len(x):
            x["relative_time"] = x["step"] / nsteps
        else:
            x["relative_time"] = 0.0
        x["product"] = p
        x["world"] = wname
        x["tournament"] = tname
        product_info.append(x)

    world_info = stats.loc[:, world_stat_names].reset_index().rename(columns=dict(index="step"))
    if len(world_info):
        world_info["relative_time"] = world_info["step"] / nsteps
    else:
        world_info["relative_time"] = 0.0
    world_info["world"] = wname
    world_info["tournament"] = tname

    return (
        pd.concat(agents_info, ignore_index=True),
        pd.concat(product_info, ignore_index=True),
        world_info,
        contracts,
        contract_stats,
        negotiations,
        offers,
        negotiation_stats,
        breaches,
    )


def get_basic_world_info(path, tname):
    stats = pd.read_csv(path / STATS_FILE, index_col=0).to_dict("list")
    adata = json.load(open(path / AGENTS_JSON_FILE))
    winfo = json.load(open(path / INFO_FILE))
    worlds = [dict(name=path.name, tournament=tname, tournament_indx=0
    , path=path, n_steps=winfo["n_steps"])]
    agents = []
    definfo = winfo.get("is_default", None)
    if definfo:
        is_default = dict(zip(winfo["agent_initial_balances"].keys(), definfo))
    else:
        is_default = dict(zip(winfo["agent_initial_balances"].keys(), itertools.repeat(False)))
    for i, (aname, info) in enumerate(adata.items()):
        if f"score_{aname}" not in stats.keys():
            continue
        score = stats[f"score_{aname}"][-1]
        aginfo = winfo["agent_profiles"][aname]
        aginfo["initial_balance"] = winfo.get("agent_initial_balances", dict()).get(aname, float("nan"))
        aginfo["is_default"] = is_default.get(aname, True)
        if "costs" in aginfo.keys():
            aginfo["cost"] = float(np.asarray(aginfo["costs"]).min())
            del aginfo["costs"]
        dd = dict(id=i, name=aname, world=worlds[0]["name"], tournament=tname, final_score=score, type=info["type"])
        dd = {**dd, **aginfo}
        agents.append( dd)
    return worlds, pd.DataFrame.from_records(agents)


def get_data(base_folder):
    base_folder = Path(base_folder)
    tournaments, worlds, agents, agent_stats, product_stats, world_stats = (
        [],
        [],
        [],
        [],
        [],
        [],
    )
    contracts, contract_stats, breaches = [], [], []
    negotiations, offers, neg_stats = [], [], []
    if is_tournament(base_folder):
        paths = get_torunaments(base_folder)
    elif is_world(base_folder):
        paths = [None]
    else:
        raise ValueError(f"Folder {str(base_folder)} contains neither tournament nor world logs")
    for i, t in enumerate(paths):
        indx = i + 1
        base_indx = (i + 1) * 1_000_000
        if t is not None:
            print(f"Processing {t.name} [{i} of {len(paths)}]", flush=True)
            tournaments.append(dict(id=indx, path=t, name=t.name))
            w, a = parse_tournament(t, indx, base_indx)
            tname = t.name
        else:
            print(f"No tournament found ... reading world info")
            tname = "none"
            tournaments.append(dict(id=tname, path=base_folder.parent, name=tname))
            w, a = get_basic_world_info(base_folder, tname)
        for j, world in enumerate(w):
            print(f"\tWorld {world['name']} [{j} of {len(w)}]", flush=True)
            wagents = a.loc[a.world == world["name"]]
            ag, pr, wo, co, cs, ng, of, ns, br = parse_world(
                Path(world["path"]),
                tname,
                world["name"],
                world["n_steps"],
                wagents,
                base_indx + j + 1,
                base_indx + j + 1,
            )
            if ag is not None and len(ag):
                agent_stats.append(ag)
            if pr is not None and len(pr):
                product_stats.append(pr)
            if wo is not None and len(wo):
                world_stats.append(wo)
            if co is not None and len(co):
                contracts.append(co)
            if cs is not None and len(cs):
                contract_stats.append(cs)
            if ng is not None and len(ng):
                negotiations.append(ng)
            if of is not None and len(of):
                offers.append(of)
            if ns is not None and len(ns):
                neg_stats.append(ns)
            if br is not None and len(br):
                breaches.append(br)
        if w is not None and len(w):
            worlds.append(pd.DataFrame.from_records(w))
        if a is not None and len(a):
            agents.append(a)

    tournaments = pd.DataFrame.from_records(tournaments)
    if worlds is not None and len(worlds):
        worlds = pd.concat(worlds, ignore_index=True)
    if agents is not None and len(agents):
        agents = pd.concat(agents, ignore_index=True)
    if agent_stats is not None and len(agent_stats):
        agent_stats = pd.concat(agent_stats, ignore_index=True)
    if product_stats is not None and len(product_stats):
        product_stats = pd.concat(product_stats, ignore_index=True)
    if world_stats is not None and len(world_stats):
        world_stats = pd.concat(world_stats, ignore_index=True)
    if contracts is not None and len(contracts):
        contracts = pd.concat(contracts, ignore_index=True)
    if contract_stats is not None and len(contract_stats):
        contract_stats = pd.concat(contract_stats, ignore_index=True)
    if negotiations is not None and len(negotiations):
        negotiations = pd.concat(negotiations, ignore_index=True)
    if offers is not None and len(offers):
        offers = pd.concat(offers, ignore_index=True)
    if neg_stats is not None and len(neg_stats):
        neg_stats = pd.concat(neg_stats, ignore_index=True)
    if breaches is not None and len(breaches):
        breaches = pd.concat(breaches, ignore_index=True)
    return (
        tournaments,
        worlds,
        agents,
        agent_stats,
        product_stats,
        world_stats,
        contracts,
        contract_stats,
        negotiations,
        offers,
        neg_stats,
        breaches,
    )


def main(folder: Path, max_worlds: Optional[int]):
    folder = Path(folder)
    if max_worlds is None:
        max_worlds = float("inf")
    dst_folder = folder / VISDATA_FOLDER
    if dst_folder.exists():
        raise ValueError(f"Destiantion folder {dst_folder} exists. Delete it if you want to recompile visualization data")
    folder = Path(folder)
    (
        tournaments,
        worlds,
        agents,
        agent_stats,
        product_stats,
        world_stats,
        contracts,
        contract_stats,
        negotiations,
        offers,
        neg_stats,
        breaches,
    ) = get_data(folder)
    dst_folder = folder / VISDATA_FOLDER
    dst_folder.mkdir(parents=True, exist_ok=True)
    for df, name in zip(
        (
            tournaments,
            worlds,
            agents,
            agent_stats,
            product_stats,
            world_stats,
            contracts,
            contract_stats,
            negotiations,
            offers,
            neg_stats,
            breaches,
        ),
        (
            "tournaments",
            "worlds",
            "agents",
            "agent_stats",
            "product_stats",
            "world_stats",
            "contracts",
            "contract_stats",
            "negotiations",
            "offers",
            "neg_stats",
            "breaches",
        ),
    ):
        if df is None or len(df) == 0:
            continue
        df = adjust_type_names(df)
        df.to_csv(dst_folder / f"{name}.csv", index=False)


def has_visdata(folder: Path):
    folder = Path(folder)
    files = (
        "tournaments",
        "worlds",
        "agents",
        "agent_stats",
        "product_stats",
        "world_stats",
    )
    if folder.name != VISDATA_FOLDER:
        folder /= VISDATA_FOLDER
    if not folder.exists():
        return False
    for file in files:
        if not nonzero(folder / f"{file}.csv"):
            return False
    return True


if __name__ == "__main__":
    import sys

    for arg in sys.argv[1:]:
        if arg == "--map":
            PATHMAP = {"/export/home": "/Users"}
        elif arg.startswith("n="):
            MAXWORLDS = int(arg.split("=")[-1])
        else:
            path = arg
    main(Path(path), MAXWORLDS)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['scml_vis', 'scml_vis.vendor', 'scml_vis.vendor.quick']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.15.4,<6.0.0',
 'click-config-file>=0.6.0,<0.7.0',
 'click>=7.1.2,<8.0.0',
 'csvs-to-sqlite>=1.2,<2.0',
 'datasette-vega>=0.6.2,<0.7.0',
 'numpy>=1.20.2,<2.0.0',
 'pandas>=1.2.3,<2.0.0',
 'plotly>=4.14.3,<5.0.0',
 'seaborn>=0.11.1,<0.12.0',
 'streamlit>=0.80.0,<0.81.0',
 'watchdog>=2.0.2,<3.0.0']

entry_points = \
{'console_scripts': ['scml-vis = scml_vis.cli:main',
                     'scmlv = scml_vis.cli:main',
                     'scmlvis = scml_vis.cli:main']}

setup_kwargs = {
    'name': 'scml-vis',
    'version': '0.2.5',
    'description': 'A simple visualiser for SCML worlds and tournaments',
    'long_description': "# scml-vis\n\n[![ci](https://github.com/yasserfarouk/scml-vis/actions/workflows/main.yml/badge.svg)](https://github.com/yasserfarouk/scml-vis/actions/workflows/main.yml)\n[![documentation](https://img.shields.io/badge/docs-mkdocs%20material-blue.svg?style=flat)](https://scml-vis.github.io/scml-vis/)\n[![pypi version](https://img.shields.io/pypi/v/scml-vis.svg)](https://pypi.org/project/scml-vis/)\n[![gitter](https://badges.gitter.im/join%20chat.svg)](https://gitter.im/scml-vis/community)\n\nA simple visualiser for SCML worlds and tournaments\n\n## Screenshots\n![Screen Shot 1](docs/shot1.png)\n![Screen Shot 2](docs/shot2.png)\n\n## Main Features\n\n- Displays any world/tournament run using the [SCML package](https://www.github.com/yasserfarouk/scml)\n- Allows filtering using worlds, agent types, and agent instances\n- Shows world statistics, agent type and instance statistics and contract \n  statistics as functions of simulation step/time\n\n## TODO List (Good Ideas for PRs)\n\n- ~~Show negotiation logs (i.e. negotiation results)~~\n- ~~Display all contracts (i.e. in a table) based on selection criteria~~\n- ~~Zoom on negotiation details (i.e. exchanged offers)~~\n- ~~Add dynamic figures using plotly/altair~~\n- ~~Add networkx like graphs of contracts / negotiations / offers~~\n- ~~Allow starting the app without specifying a folder.~~\n- Add saving and loading of the visualizer's state (i.e. what is visible).\n- Add new figure types that do not have time/step in the x-axis.\n- Correcting the placement of weights on edges in network views.\n- Adding a graph showing negotiation history in the ufun-space of negotiators (will require a change in the scml package).\n- Resolving the strange behavior of CI bands in plotly in some cases.\n\n## Requirements\n\nscml-vis requires Python 3.8 or above.\n\n## Installation\n\nWith `pip`:\n```bash\npython3 -m pip install scml-vis\n```\n\nWith [`pipx`](https://github.com/pipxproject/pipx):\n```bash\npython3 -m pip install --user pipx\n\npipx install scml-vis\n```\n",
    'author': 'Yasser Mohammad',
    'author_email': 'yasserfarouk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/scml-vis/scml-vis',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
